# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

from typing import TYPE_CHECKING
from ..call import call, call_async

from ..protobufs import main_pb2
from ..units import Units

if TYPE_CHECKING:
    from .device import Device


class DeviceSettings:
    """
    Class providing access to various device settings and properties.
    """

    def __init__(self, device: 'Device'):
        self._device = device

    def get(
            self,
            setting: str
    ) -> float:
        """
        Returns any device setting or property.

        Args:
            setting: Name of the setting.

        Returns:
            Setting value.
        """
        request = main_pb2.DeviceGetSettingRequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.setting = setting
        response = main_pb2.DeviceGetSettingResponse()
        call("device/get_device_setting", request, response)
        return response.value

    async def get_async(
            self,
            setting: str
    ) -> float:
        """
        Returns any device setting or property.

        Args:
            setting: Name of the setting.

        Returns:
            Setting value.
        """
        request = main_pb2.DeviceGetSettingRequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.setting = setting
        response = main_pb2.DeviceGetSettingResponse()
        await call_async("device/get_device_setting", request, response)
        return response.value

    def set(
            self,
            setting: str,
            value: float,
            unit: Units = Units.NATIVE
    ) -> None:
        """
        Sets any device setting.

        Args:
            setting: Name of the setting.
            value: Value of the setting.
            unit: Units of setting.
        """
        request = main_pb2.DeviceSetSettingRequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.setting = setting
        request.value = value
        request.unit = unit.value
        call("device/set_device_setting", request)

    async def set_async(
            self,
            setting: str,
            value: float,
            unit: Units = Units.NATIVE
    ) -> None:
        """
        Sets any device setting.

        Args:
            setting: Name of the setting.
            value: Value of the setting.
            unit: Units of setting.
        """
        request = main_pb2.DeviceSetSettingRequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.setting = setting
        request.value = value
        request.unit = unit.value
        await call_async("device/set_device_setting", request)



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
# pylint: disable=W0201

from ..protobufs import main_pb2


class DeviceIOInfo:
    """
    Class representing information on the I/O channels of the device.
    """

    @property
    def number_analog_outputs(self) -> int:
        """
        Number of analog output channels.
        """

        return self._number_analog_outputs

    @number_analog_outputs.setter
    def number_analog_outputs(self, value: int) -> None:
        self._number_analog_outputs = value

    @property
    def number_analog_inputs(self) -> int:
        """
        Number of analog input channels.
        """

        return self._number_analog_inputs

    @number_analog_inputs.setter
    def number_analog_inputs(self, value: int) -> None:
        self._number_analog_inputs = value

    @property
    def number_digital_outputs(self) -> int:
        """
        Number of digital output channels.
        """

        return self._number_digital_outputs

    @number_digital_outputs.setter
    def number_digital_outputs(self, value: int) -> None:
        self._number_digital_outputs = value

    @property
    def number_digital_inputs(self) -> int:
        """
        Number of digital input channels.
        """

        return self._number_digital_inputs

    @number_digital_inputs.setter
    def number_digital_inputs(self, value: int) -> None:
        self._number_digital_inputs = value

    def __repr__(self) -> str:
        return str(self.__dict__)

    @staticmethod
    def from_protobuf(pb_data: main_pb2.DeviceIOInfo) -> 'DeviceIOInfo':
        instance = DeviceIOInfo.__new__(DeviceIOInfo)  # type: DeviceIOInfo
        instance.number_analog_outputs = pb_data.number_analog_outputs
        instance.number_analog_inputs = pb_data.number_analog_inputs
        instance.number_digital_outputs = pb_data.number_digital_outputs
        instance.number_digital_inputs = pb_data.number_digital_inputs
        return instance



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
from enum import Enum


class DeviceType(Enum):
    """
    Denotes type of an device and units it accepts.
    """

    UNKNOWN = 0
    LINEAR = 1
    ROTARY = 2



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #


class SettingConstants:

    """Accel."""
    ACCEL = "accel"

    """Calibration Type."""
    CALIBRATION_TYPE = "calibration.type"

    """Cloop Continuous Enable."""
    CLOOP_CONTINUOUS_ENABLE = "cloop.continuous.enable"

    """Cloop Counts."""
    CLOOP_COUNTS = "cloop.counts"

    """Cloop Displace Tolerance."""
    CLOOP_DISPLACE_TOLERANCE = "cloop.displace.tolerance"

    """Cloop Duration Max."""
    CLOOP_DURATION_MAX = "cloop.duration.max"

    """Cloop Enable."""
    CLOOP_ENABLE = "cloop.enable"

    """Cloop Mode."""
    CLOOP_MODE = "cloop.mode"

    """Cloop Recovery Enable."""
    CLOOP_RECOVERY_ENABLE = "cloop.recovery.enable"

    """Cloop Settle Period."""
    CLOOP_SETTLE_PERIOD = "cloop.settle.period"

    """Cloop Settle Tolerance."""
    CLOOP_SETTLE_TOLERANCE = "cloop.settle.tolerance"

    """Cloop Stalltimeout."""
    CLOOP_STALLTIMEOUT = "cloop.stalltimeout"

    """Cloop Steps."""
    CLOOP_STEPS = "cloop.steps"

    """Cloop Timeout."""
    CLOOP_TIMEOUT = "cloop.timeout"

    """Comm Address."""
    COMM_ADDRESS = "comm.address"

    """Comm Alert."""
    COMM_ALERT = "comm.alert"

    """Comm Checksum."""
    COMM_CHECKSUM = "comm.checksum"

    """Comm Protocol."""
    COMM_PROTOCOL = "comm.protocol"

    """Comm Rs 232 Baud."""
    COMM_RS_232_BAUD = "comm.rs232.baud"

    """Comm Rs 232 Protocol."""
    COMM_RS_232_PROTOCOL = "comm.rs232.protocol"

    """Comm Rs 485 Baud."""
    COMM_RS_485_BAUD = "comm.rs485.baud"

    """Comm Rs 485 Enable."""
    COMM_RS_485_ENABLE = "comm.rs485.enable"

    """Comm Rs 485 Protocol."""
    COMM_RS_485_PROTOCOL = "comm.rs485.protocol"

    """Comm Usb Protocol."""
    COMM_USB_PROTOCOL = "comm.usb.protocol"

    """Deviceid."""
    DEVICEID = "deviceid"

    """Driver Current Hold."""
    DRIVER_CURRENT_HOLD = "driver.current.hold"

    """Driver Current Max."""
    DRIVER_CURRENT_MAX = "driver.current.max"

    """Driver Current Run."""
    DRIVER_CURRENT_RUN = "driver.current.run"

    """Driver Current Servo."""
    DRIVER_CURRENT_SERVO = "driver.current.servo"

    """Driver Dir."""
    DRIVER_DIR = "driver.dir"

    """Driver Enabled."""
    DRIVER_ENABLED = "driver.enabled"

    """Driver Temperature."""
    DRIVER_TEMPERATURE = "driver.temperature"

    """Encoder Count."""
    ENCODER_COUNT = "encoder.count"

    """Encoder Count Cal."""
    ENCODER_COUNT_CAL = "encoder.count.cal"

    """Encoder Count Calibrated."""
    ENCODER_COUNT_CALIBRATED = "encoder.count.calibrated"

    """Encoder Counth."""
    ENCODER_COUNTH = "encoder.counth"

    """Encoder Dir."""
    ENCODER_DIR = "encoder.dir"

    """Encoder Error."""
    ENCODER_ERROR = "encoder.error"

    """Encoder Fault Type."""
    ENCODER_FAULT_TYPE = "encoder.fault.type"

    """Encoder Filter."""
    ENCODER_FILTER = "encoder.filter"

    """Encoder Index Count."""
    ENCODER_INDEX_COUNT = "encoder.index.count"

    """Encoder Index Mode."""
    ENCODER_INDEX_MODE = "encoder.index.mode"

    """Encoder Index Phase."""
    ENCODER_INDEX_PHASE = "encoder.index.phase"

    """Encoder Mode."""
    ENCODER_MODE = "encoder.mode"

    """Encoder Pos."""
    ENCODER_POS = "encoder.pos"

    """Encoder Pos Error."""
    ENCODER_POS_ERROR = "encoder.pos.error"

    """Filter Holderid."""
    FILTER_HOLDERID = "filter.holderid"

    """Force Average."""
    FORCE_AVERAGE = "force.average"

    """Joy Debug."""
    JOY_DEBUG = "joy.debug"

    """Knob Dir."""
    KNOB_DIR = "knob.dir"

    """Knob Distance."""
    KNOB_DISTANCE = "knob.distance"

    """Knob Enable."""
    KNOB_ENABLE = "knob.enable"

    """Knob Force."""
    KNOB_FORCE = "knob.force"

    """Knob Forceprofile."""
    KNOB_FORCEPROFILE = "knob.forceprofile"

    """Knob Maxspeed."""
    KNOB_MAXSPEED = "knob.maxspeed"

    """Knob Mode."""
    KNOB_MODE = "knob.mode"

    """Knob Speedprofile."""
    KNOB_SPEEDPROFILE = "knob.speedprofile"

    """Lamp Current."""
    LAMP_CURRENT = "lamp.current"

    """Lamp Current Max."""
    LAMP_CURRENT_MAX = "lamp.current.max"

    """Lamp Flux."""
    LAMP_FLUX = "lamp.flux"

    """Lamp Flux Max."""
    LAMP_FLUX_MAX = "lamp.flux.max"

    """Lamp Status."""
    LAMP_STATUS = "lamp.status"

    """Lamp Temperature."""
    LAMP_TEMPERATURE = "lamp.temperature"

    """Lamp Wavelength Fwhm."""
    LAMP_WAVELENGTH_FWHM = "lamp.wavelength.fwhm"

    """Lamp Wavelength Peak."""
    LAMP_WAVELENGTH_PEAK = "lamp.wavelength.peak"

    """Limit Approach Accel."""
    LIMIT_APPROACH_ACCEL = "limit.approach.accel"

    """Limit Approach Maxspeed."""
    LIMIT_APPROACH_MAXSPEED = "limit.approach.maxspeed"

    """Limit Away Action."""
    LIMIT_AWAY_ACTION = "limit.away.action"

    """Limit Away Edge."""
    LIMIT_AWAY_EDGE = "limit.away.edge"

    """Limit Away Pos."""
    LIMIT_AWAY_POS = "limit.away.pos"

    """Limit Away Posupdate."""
    LIMIT_AWAY_POSUPDATE = "limit.away.posupdate"

    """Limit Away Preset."""
    LIMIT_AWAY_PRESET = "limit.away.preset"

    """Limit Away State."""
    LIMIT_AWAY_STATE = "limit.away.state"

    """Limit Away Triggered."""
    LIMIT_AWAY_TRIGGERED = "limit.away.triggered"

    """Limit Away Type."""
    LIMIT_AWAY_TYPE = "limit.away.type"

    """Limit C Action."""
    LIMIT_C_ACTION = "limit.c.action"

    """Limit C Edge."""
    LIMIT_C_EDGE = "limit.c.edge"

    """Limit C Pos."""
    LIMIT_C_POS = "limit.c.pos"

    """Limit C Posupdate."""
    LIMIT_C_POSUPDATE = "limit.c.posupdate"

    """Limit C Preset."""
    LIMIT_C_PRESET = "limit.c.preset"

    """Limit C State."""
    LIMIT_C_STATE = "limit.c.state"

    """Limit C Triggered."""
    LIMIT_C_TRIGGERED = "limit.c.triggered"

    """Limit C Type."""
    LIMIT_C_TYPE = "limit.c.type"

    """Limit Cycle Dist."""
    LIMIT_CYCLE_DIST = "limit.cycle.dist"

    """Limit D Action."""
    LIMIT_D_ACTION = "limit.d.action"

    """Limit D Edge."""
    LIMIT_D_EDGE = "limit.d.edge"

    """Limit D Pos."""
    LIMIT_D_POS = "limit.d.pos"

    """Limit D Posupdate."""
    LIMIT_D_POSUPDATE = "limit.d.posupdate"

    """Limit D Preset."""
    LIMIT_D_PRESET = "limit.d.preset"

    """Limit D State."""
    LIMIT_D_STATE = "limit.d.state"

    """Limit D Triggered."""
    LIMIT_D_TRIGGERED = "limit.d.triggered"

    """Limit D Type."""
    LIMIT_D_TYPE = "limit.d.type"

    """Limit Detect Decelonly."""
    LIMIT_DETECT_DECELONLY = "limit.detect.decelonly"

    """Limit Detect Maxspeed."""
    LIMIT_DETECT_MAXSPEED = "limit.detect.maxspeed"

    """Limit Home Action."""
    LIMIT_HOME_ACTION = "limit.home.action"

    """Limit Home Edge."""
    LIMIT_HOME_EDGE = "limit.home.edge"

    """Limit Home Pos."""
    LIMIT_HOME_POS = "limit.home.pos"

    """Limit Home Posupdate."""
    LIMIT_HOME_POSUPDATE = "limit.home.posupdate"

    """Limit Home Preset."""
    LIMIT_HOME_PRESET = "limit.home.preset"

    """Limit Home State."""
    LIMIT_HOME_STATE = "limit.home.state"

    """Limit Home Triggered."""
    LIMIT_HOME_TRIGGERED = "limit.home.triggered"

    """Limit Home Type."""
    LIMIT_HOME_TYPE = "limit.home.type"

    """Limit Max."""
    LIMIT_MAX = "limit.max"

    """Limit Min."""
    LIMIT_MIN = "limit.min"

    """Limit Start Pos."""
    LIMIT_START_POS = "limit.start.pos"

    """Limit Swapinputs."""
    LIMIT_SWAPINPUTS = "limit.swapinputs"

    """Lockstep Numgroups."""
    LOCKSTEP_NUMGROUPS = "lockstep.numgroups"

    """Lockstep Tolerance."""
    LOCKSTEP_TOLERANCE = "lockstep.tolerance"

    """Maxspeed."""
    MAXSPEED = "maxspeed"

    """Motion Accelonly."""
    MOTION_ACCELONLY = "motion.accelonly"

    """Motion Decelonly."""
    MOTION_DECELONLY = "motion.decelonly"

    """Motion Index Dist."""
    MOTION_INDEX_DIST = "motion.index.dist"

    """Motion Index Num."""
    MOTION_INDEX_NUM = "motion.index.num"

    """Motor Current Max."""
    MOTOR_CURRENT_MAX = "motor.current.max"

    """Parking State."""
    PARKING_STATE = "parking.state"

    """Peripheral Serial."""
    PERIPHERAL_SERIAL = "peripheral.serial"

    """Peripheralid."""
    PERIPHERALID = "peripheralid"

    """Pos."""
    POS = "pos"

    """Resolution."""
    RESOLUTION = "resolution"

    """Scope Delay."""
    SCOPE_DELAY = "scope.delay"

    """Scope Timebase."""
    SCOPE_TIMEBASE = "scope.timebase"

    """Stream Numbufs."""
    STREAM_NUMBUFS = "stream.numbufs"

    """Stream Numstreams."""
    STREAM_NUMSTREAMS = "stream.numstreams"

    """System Access."""
    SYSTEM_ACCESS = "system.access"

    """System Axiscount."""
    SYSTEM_AXISCOUNT = "system.axiscount"

    """System Current."""
    SYSTEM_CURRENT = "system.current"

    """System Led Enable."""
    SYSTEM_LED_ENABLE = "system.led.enable"

    """System Serial."""
    SYSTEM_SERIAL = "system.serial"

    """System Temperature."""
    SYSTEM_TEMPERATURE = "system.temperature"

    """System Voltage."""
    SYSTEM_VOLTAGE = "system.voltage"

    """Trigger Numactions."""
    TRIGGER_NUMACTIONS = "trigger.numactions"

    """Trigger Numtriggers."""
    TRIGGER_NUMTRIGGERS = "trigger.numtriggers"

    """Vel."""
    VEL = "vel"

    """Version."""
    VERSION = "version"

    """Version Build."""
    VERSION_BUILD = "version.build"

    """Virtual Numvirtual."""
    VIRTUAL_NUMVIRTUAL = "virtual.numvirtual"



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

# pylint: disable=too-many-arguments

from typing import TYPE_CHECKING
from ..call import call, call_async, call_sync

from ..protobufs import main_pb2
from .binary_settings import BinarySettings
from .device_identity import DeviceIdentity
from .device_settings import DeviceSettings
from .command_code import CommandCode
from .message import Message
from ..units import Units
from .device_type import DeviceType
from ..firmware_version import FirmwareVersion

if TYPE_CHECKING:
    from .connection import Connection


class Device:
    """
    Represents a device using the binary protocol.
    """

    DEFAULT_MOVEMENT_TIMEOUT = 60
    """
    Default timeout for move commands in seconds.
    """

    @property
    def connection(self) -> 'Connection':
        """
        Connection of this device.
        """
        return self._connection

    @property
    def settings(self) -> DeviceSettings:
        """
        Settings and properties of this axis.
        """
        return self._settings

    @property
    def device_address(self) -> int:
        """
        The device address uniquely identifies the device on the connection.
        It can be configured or automatically assigned by the renumber command.
        """
        return self._device_address

    @property
    def identity(self) -> DeviceIdentity:
        """
        Identity of the device.
        """
        return self.__retrieve_identity()

    @property
    def device_id(self) -> int:
        """
        Unique ID of the device hardware.
        """
        return self.identity.device_id

    @property
    def serial_number(self) -> int:
        """
        Serial number of the device.
        """
        return self.identity.serial_number

    @property
    def name(self) -> str:
        """
        Name of the product.
        """
        return self.identity.name

    @property
    def firmware_version(self) -> FirmwareVersion:
        """
        Version of the firmware.
        """
        return self.identity.firmware_version

    @property
    def is_peripheral(self) -> bool:
        """
        Indicates whether the device is a peripheral or part of an integrated device.
        """
        return self.identity.is_peripheral

    @property
    def peripheral_id(self) -> int:
        """
        Unique ID of the peripheral hardware.
        """
        return self.identity.peripheral_id

    @property
    def peripheral_name(self) -> str:
        """
        Name of the peripheral hardware.
        """
        return self.identity.peripheral_name

    @property
    def device_type(self) -> DeviceType:
        """
        Determines the type of an device and units it accepts.
        """
        return self.identity.device_type

    def __init__(self, connection: 'Connection', device_address: int):
        self._connection = connection
        self._settings = DeviceSettings(self)
        self._device_address = device_address

    def generic_command(
            self,
            command: CommandCode,
            data: int = 0,
            timeout: float = 0.0,
            check_errors: bool = True
    ) -> Message:
        """
        Sends a generic Binary command to this device.
        For more information please refer to the
        [Binary Protocol Manual](https://www.zaber.com/protocol-manual?protocol=Binary#topic_quick_command_reference).

        Args:
            command: Command to send.
            data: Optional data argument to the command. Defaults to zero.
            timeout: Number of seconds to wait for a response from the device. 0 or negative defaults to 0.5s.
            check_errors: Controls whether to throw an exception when the device rejects the command.

        Returns:
            A response to the command.
        """
        request = main_pb2.GenericBinaryRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.command = command.value
        request.data = data
        request.timeout = timeout
        request.check_errors = check_errors
        response = main_pb2.BinaryMessage()
        call("binary/interface/generic_command", request, response)
        return Message.from_protobuf(response)

    async def generic_command_async(
            self,
            command: CommandCode,
            data: int = 0,
            timeout: float = 0.0,
            check_errors: bool = True
    ) -> Message:
        """
        Sends a generic Binary command to this device.
        For more information please refer to the
        [Binary Protocol Manual](https://www.zaber.com/protocol-manual?protocol=Binary#topic_quick_command_reference).

        Args:
            command: Command to send.
            data: Optional data argument to the command. Defaults to zero.
            timeout: Number of seconds to wait for a response from the device. 0 or negative defaults to 0.5s.
            check_errors: Controls whether to throw an exception when the device rejects the command.

        Returns:
            A response to the command.
        """
        request = main_pb2.GenericBinaryRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.command = command.value
        request.data = data
        request.timeout = timeout
        request.check_errors = check_errors
        response = main_pb2.BinaryMessage()
        await call_async("binary/interface/generic_command", request, response)
        return Message.from_protobuf(response)

    def generic_command_no_response(
            self,
            command: CommandCode,
            data: int = 0
    ) -> None:
        """
        Sends a generic Binary command to this device without expecting a response.
        For more information please refer to the
        [Binary Protocol Manual](https://www.zaber.com/protocol-manual?protocol=Binary#topic_quick_command_reference).

        Args:
            command: Command to send.
            data: Optional data argument to the command. Defaults to zero.
        """
        request = main_pb2.GenericBinaryRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.command = command.value
        request.data = data
        call("binary/interface/generic_command_no_response", request)

    async def generic_command_no_response_async(
            self,
            command: CommandCode,
            data: int = 0
    ) -> None:
        """
        Sends a generic Binary command to this device without expecting a response.
        For more information please refer to the
        [Binary Protocol Manual](https://www.zaber.com/protocol-manual?protocol=Binary#topic_quick_command_reference).

        Args:
            command: Command to send.
            data: Optional data argument to the command. Defaults to zero.
        """
        request = main_pb2.GenericBinaryRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.command = command.value
        request.data = data
        await call_async("binary/interface/generic_command_no_response", request)

    def generic_command_with_units(
            self,
            command: CommandCode,
            data: float,
            from_unit: Units = Units.NATIVE,
            to_unit: Units = Units.NATIVE,
            timeout: float = 0.0
    ) -> float:
        """
        Sends a generic Binary command to this device with unit conversions for both sent data and retrieved data.

        Args:
            command: Command to send.
            data: Data argument to the command. Set to zero if command does not require argument.
            from_unit: Unit to convert sent data from.
            to_unit: Unit to convert retrieved data to.
            timeout: Number of seconds to wait for a response from the device. 0 or negative defaults to 0.5s.

        Returns:
            Data that has been converted to the provided unit.
        """
        request = main_pb2.BinaryGenericWithUnitsRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.command = command.value
        request.data = data
        request.from_unit = from_unit.value
        request.to_unit = to_unit.value
        request.timeout = timeout
        response = main_pb2.BinaryGenericWithUnitsResponse()
        call("binary/device/generic_command_with_units", request, response)
        return response.data

    async def generic_command_with_units_async(
            self,
            command: CommandCode,
            data: float,
            from_unit: Units = Units.NATIVE,
            to_unit: Units = Units.NATIVE,
            timeout: float = 0.0
    ) -> float:
        """
        Sends a generic Binary command to this device with unit conversions for both sent data and retrieved data.

        Args:
            command: Command to send.
            data: Data argument to the command. Set to zero if command does not require argument.
            from_unit: Unit to convert sent data from.
            to_unit: Unit to convert retrieved data to.
            timeout: Number of seconds to wait for a response from the device. 0 or negative defaults to 0.5s.

        Returns:
            Data that has been converted to the provided unit.
        """
        request = main_pb2.BinaryGenericWithUnitsRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.command = command.value
        request.data = data
        request.from_unit = from_unit.value
        request.to_unit = to_unit.value
        request.timeout = timeout
        response = main_pb2.BinaryGenericWithUnitsResponse()
        await call_async("binary/device/generic_command_with_units", request, response)
        return response.data

    def home(
            self,
            unit: Units = Units.NATIVE,
            timeout: float = DEFAULT_MOVEMENT_TIMEOUT
    ) -> float:
        """
        Homes device. Device returns to its homing position.

        Args:
            unit: Unit to convert returned position to.
            timeout: Number of seconds to wait for response from the device chain (defaults to 60s).

        Returns:
            Current position that has been converted to the provided unit.
        """
        request = main_pb2.BinaryDeviceHomeRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.unit = unit.value
        request.timeout = timeout
        response = main_pb2.BinaryDeviceMovementResponse()
        call("binary/device/home", request, response)
        return response.data

    async def home_async(
            self,
            unit: Units = Units.NATIVE,
            timeout: float = DEFAULT_MOVEMENT_TIMEOUT
    ) -> float:
        """
        Homes device. Device returns to its homing position.

        Args:
            unit: Unit to convert returned position to.
            timeout: Number of seconds to wait for response from the device chain (defaults to 60s).

        Returns:
            Current position that has been converted to the provided unit.
        """
        request = main_pb2.BinaryDeviceHomeRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.unit = unit.value
        request.timeout = timeout
        response = main_pb2.BinaryDeviceMovementResponse()
        await call_async("binary/device/home", request, response)
        return response.data

    def stop(
            self,
            unit: Units = Units.NATIVE,
            timeout: float = DEFAULT_MOVEMENT_TIMEOUT
    ) -> float:
        """
        Stops ongoing device movement. Decelerates until zero speed.

        Args:
            unit: Unit to convert returned position to.
            timeout: Number of seconds to wait for response from the device chain (defaults to 60s).

        Returns:
            Current position that has been converted to the provided unit.
        """
        request = main_pb2.BinaryDeviceStopRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.unit = unit.value
        request.timeout = timeout
        response = main_pb2.BinaryDeviceMovementResponse()
        call("binary/device/stop", request, response)
        return response.data

    async def stop_async(
            self,
            unit: Units = Units.NATIVE,
            timeout: float = DEFAULT_MOVEMENT_TIMEOUT
    ) -> float:
        """
        Stops ongoing device movement. Decelerates until zero speed.

        Args:
            unit: Unit to convert returned position to.
            timeout: Number of seconds to wait for response from the device chain (defaults to 60s).

        Returns:
            Current position that has been converted to the provided unit.
        """
        request = main_pb2.BinaryDeviceStopRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.unit = unit.value
        request.timeout = timeout
        response = main_pb2.BinaryDeviceMovementResponse()
        await call_async("binary/device/stop", request, response)
        return response.data

    def move_absolute(
            self,
            position: float,
            unit: Units = Units.NATIVE,
            timeout: float = DEFAULT_MOVEMENT_TIMEOUT
    ) -> float:
        """
        Move device to absolute position.

        Args:
            position: Absolute position.
            unit: Unit for the provided position as well as position returned by the device.
            timeout: Number of seconds to wait for response from the device chain (defaults to 60s).

        Returns:
            Current position that has been converted to the provided unit.
        """
        request = main_pb2.BinaryDeviceMoveRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.type = main_pb2.BinaryDeviceMoveRequest.ABS
        request.arg = position
        request.unit = unit.value
        request.timeout = timeout
        response = main_pb2.BinaryDeviceMovementResponse()
        call("binary/device/move", request, response)
        return response.data

    async def move_absolute_async(
            self,
            position: float,
            unit: Units = Units.NATIVE,
            timeout: float = DEFAULT_MOVEMENT_TIMEOUT
    ) -> float:
        """
        Move device to absolute position.

        Args:
            position: Absolute position.
            unit: Unit for the provided position as well as position returned by the device.
            timeout: Number of seconds to wait for response from the device chain (defaults to 60s).

        Returns:
            Current position that has been converted to the provided unit.
        """
        request = main_pb2.BinaryDeviceMoveRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.type = main_pb2.BinaryDeviceMoveRequest.ABS
        request.arg = position
        request.unit = unit.value
        request.timeout = timeout
        response = main_pb2.BinaryDeviceMovementResponse()
        await call_async("binary/device/move", request, response)
        return response.data

    def move_relative(
            self,
            position: float,
            unit: Units = Units.NATIVE,
            timeout: float = DEFAULT_MOVEMENT_TIMEOUT
    ) -> float:
        """
        Move device to position relative to current position.

        Args:
            position: Relative position.
            unit: Unit for the provided position as well as position returned by the device.
            timeout: Number of seconds to wait for response from the device chain (defaults to 60s).

        Returns:
            Current position that has been converted to the provided unit.
        """
        request = main_pb2.BinaryDeviceMoveRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.type = main_pb2.BinaryDeviceMoveRequest.REL
        request.arg = position
        request.unit = unit.value
        request.timeout = timeout
        response = main_pb2.BinaryDeviceMovementResponse()
        call("binary/device/move", request, response)
        return response.data

    async def move_relative_async(
            self,
            position: float,
            unit: Units = Units.NATIVE,
            timeout: float = DEFAULT_MOVEMENT_TIMEOUT
    ) -> float:
        """
        Move device to position relative to current position.

        Args:
            position: Relative position.
            unit: Unit for the provided position as well as position returned by the device.
            timeout: Number of seconds to wait for response from the device chain (defaults to 60s).

        Returns:
            Current position that has been converted to the provided unit.
        """
        request = main_pb2.BinaryDeviceMoveRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.type = main_pb2.BinaryDeviceMoveRequest.REL
        request.arg = position
        request.unit = unit.value
        request.timeout = timeout
        response = main_pb2.BinaryDeviceMovementResponse()
        await call_async("binary/device/move", request, response)
        return response.data

    def move_velocity(
            self,
            velocity: float,
            unit: Units = Units.NATIVE
    ) -> float:
        """
        Begins to move device at specified speed.

        Args:
            velocity: Movement velocity.
            unit: Unit to convert returned velocity to.

        Returns:
            Device velocity that has been converted to the provided unit.
        """
        request = main_pb2.BinaryDeviceMoveRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.type = main_pb2.BinaryDeviceMoveRequest.VEL
        request.arg = velocity
        request.unit = unit.value
        response = main_pb2.BinaryDeviceMovementResponse()
        call("binary/device/move", request, response)
        return response.data

    async def move_velocity_async(
            self,
            velocity: float,
            unit: Units = Units.NATIVE
    ) -> float:
        """
        Begins to move device at specified speed.

        Args:
            velocity: Movement velocity.
            unit: Unit to convert returned velocity to.

        Returns:
            Device velocity that has been converted to the provided unit.
        """
        request = main_pb2.BinaryDeviceMoveRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.type = main_pb2.BinaryDeviceMoveRequest.VEL
        request.arg = velocity
        request.unit = unit.value
        response = main_pb2.BinaryDeviceMovementResponse()
        await call_async("binary/device/move", request, response)
        return response.data

    def wait_until_idle(
            self
    ) -> None:
        """
        Waits until device stops moving.
        """
        request = main_pb2.BinaryDeviceWaitUntilIdleRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        call("binary/device/wait_until_idle", request)

    async def wait_until_idle_async(
            self
    ) -> None:
        """
        Waits until device stops moving.
        """
        request = main_pb2.BinaryDeviceWaitUntilIdleRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        await call_async("binary/device/wait_until_idle", request)

    def is_busy(
            self
    ) -> bool:
        """
        Check whether the device is moving.

        Returns:
            True if the device is currently executing a motion command.
        """
        request = main_pb2.BinaryDeviceIsBusyRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        response = main_pb2.BinaryDeviceIsBusyResponse()
        call("binary/device/is_busy", request, response)
        return response.is_busy

    async def is_busy_async(
            self
    ) -> bool:
        """
        Check whether the device is moving.

        Returns:
            True if the device is currently executing a motion command.
        """
        request = main_pb2.BinaryDeviceIsBusyRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        response = main_pb2.BinaryDeviceIsBusyResponse()
        await call_async("binary/device/is_busy", request, response)
        return response.is_busy

    def identify(
            self
    ) -> DeviceIdentity:
        """
        Queries the device and the database, gathering information about the product.
        Without this information features such as unit conversions will not work.
        Usually, called automatically by detect devices method.

        Returns:
            Device identification data.
        """
        request = main_pb2.BinaryDeviceIdentifyRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        response = main_pb2.BinaryDeviceIdentity()
        call("binary/device/identify", request, response)
        return DeviceIdentity.from_protobuf(response)

    async def identify_async(
            self
    ) -> DeviceIdentity:
        """
        Queries the device and the database, gathering information about the product.
        Without this information features such as unit conversions will not work.
        Usually, called automatically by detect devices method.

        Returns:
            Device identification data.
        """
        request = main_pb2.BinaryDeviceIdentifyRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        response = main_pb2.BinaryDeviceIdentity()
        await call_async("binary/device/identify", request, response)
        return DeviceIdentity.from_protobuf(response)

    def get_position(
            self,
            unit: Units = Units.NATIVE
    ) -> float:
        """
        Returns current device position.

        Args:
            unit: Units of position.

        Returns:
            Axis position.
        """
        request = main_pb2.BinaryDeviceGetSettingRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.setting = BinarySettings.CURRENT_POSITION.value
        request.unit = unit.value
        response = main_pb2.BinaryDeviceGetSettingResponse()
        call("binary/device/get_setting", request, response)
        return response.value

    async def get_position_async(
            self,
            unit: Units = Units.NATIVE
    ) -> float:
        """
        Returns current device position.

        Args:
            unit: Units of position.

        Returns:
            Axis position.
        """
        request = main_pb2.BinaryDeviceGetSettingRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        request.setting = BinarySettings.CURRENT_POSITION.value
        request.unit = unit.value
        response = main_pb2.BinaryDeviceGetSettingResponse()
        await call_async("binary/device/get_setting", request, response)
        return response.value

    def __repr__(
            self
    ) -> str:
        """
        Returns a string that represents the device.

        Returns:
            A string that represents the device.
        """
        request = main_pb2.ToStringRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        response = main_pb2.ToStringResponse()
        call_sync("binary/device/device_to_string", request, response)
        return response.to_str

    def __retrieve_identity(
            self
    ) -> DeviceIdentity:
        """
        Returns identity.

        Returns:
            Device identity.
        """
        request = main_pb2.BinaryDeviceGetIdentityRequest()
        request.interface_id = self.connection.interface_id
        request.device = self.device_address
        response = main_pb2.BinaryDeviceGetIdentityResponse()
        call_sync("binary/device/get_identity", request, response)
        return DeviceIdentity.from_protobuf(response.identity)
